/*************************************************************/
/* Input the needed info for an em analysis.                 */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr23may87 */ 

#include "local.h"
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <string.h>

#include "son.h"
#include "em.typ"

#include <sys/errno.h>


/*flops info*/
/* #include "vec/flocnt.h" */

/* char *strcpy(),*strncpy(); - REMOVED for IBM6000 - KRS */
void qsort();
Rpoly *add_rpoly();
Ipoly *add_ipoly();

/* Security varaibles */
int good_key;
#define EM_SUB_LIMIT 120

extern char prog_name[];
extern int get_sub_limit();

Get_circuit(filename,Box,cir,prog_id, read_header)  
char *filename;
Box_info *Box;  
Cir *cir; 
int prog_id;
int read_header;	/* if True reads only header info */
/*************************************************************/
/* Input the needed info for an em analysis.                 */
/* Returns:						     */
/*	1  - bad key					     */
/*	0  - read file properly & good key		     */
/*	-3 - No file present				     */
/*	-2 - unexpected end of file.			     */
/*	-1 - .geo file is of improper format		     */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr23may87 */ 
/* jcr13may89 modified for polygonal outline circuit description*/
{ FILE *in;
  int i;
  port_typ *port;
  via_typ *via;
  Poly_list *plev;
  Rpoly *rpoly;
  int rc;
  int security_rc;

  /*Check time out*/
  good_key = check_security(prog_id);
  if ( good_key ) { 
	security_rc = 0;
  } else {
	security_rc = 1;
  }
  
  
  if ( ! good_key ) exit_program (1);

  if ( (in=fopen(filename,"r")) == NULL ) return(-3);  /*No file present*/

  /* The Box variable must have been initialized by init_box b4 calling 
   * this proc.
   */

  /* Initialize line counter */
  Fgets_reset();

  /* Read in complete circuit information */
  rc = read_cir(filename,in,Box,cir,read_header);
  fclose(in);
  if ( rc != 0 ) return rc;

  Fgets_done();
  if (read_header) return( security_rc );


  /* Check:
   *     1. All ports have proper level, polygon annd index
   *     2. All vias have proper level, polygon annd index
   */

  /* check ports */
  for (i=0; i < cir->ports->n; ++i) {
	port = cir->ports->port0 + i;
	if ( port->ilev > cir->n - 1 ) {
		fprintf(stderr,"File \"%s\", line %d: POR %d %d %d - level is too large\n",
			filename,port->line_nu,port->poly_index, port->ipt, port->ilev);
		return(-1); 
	}

	plev = cir->plev0 + port->ilev;
	if ( port->poly_index >= plev->n) {
		fprintf(stderr,"File \"%s\", line %d: POR %d %d %d - Polygon number is too large\n",
			filename,port->line_nu,port->poly_index, port->ipt, port->ilev);
		return(-1); 
	}

	rpoly = plev->rpoly0 + port->poly_index;
	if ( port->ipt >= rpoly->n - 1 ) {
		fprintf(stderr,"File \"%s\" line %d: POR %d %d %d - Polygon vertice index is too large\n",
			filename,port->line_nu,port->poly_index, port->ipt, port->ilev);
		return(-1); 
	}
  }

  /* check vias */
  for (i=0; i < cir->vias->n; ++i) {
	via = cir->vias->via0 + i;
	if ( via->ilev > cir->n - 1 ) {
		fprintf(stderr,"File \"%s\" line %d: VIA %d %d %d - level is too large\n",
			filename,via->line_nu,via->poly_index, via->ipt, via->ilev);
		return(-1); 
	}

	plev = cir->plev0 + via->ilev;
	if ( via->poly_index >= plev->n) {
		fprintf(stderr,"File \"%s\" line %d: VIA %d %d %d - Polygon number is too large\n",
			filename,via->line_nu,via->poly_index, via->ipt, via->ilev);
		return(-1); 
	}

	rpoly = plev->rpoly0 + via->poly_index;
	if ( via->ipt >= rpoly->n - 1 ) {
		fprintf(stderr,"File \"%s\" line %d: VIA %d %d %d - Polygon vertice index is too large\n",
			filename,via->line_nu,via->poly_index, via->ipt, via->ilev);
		return(-1); 
	}
  }

  return(security_rc);

} /*Get_circuit*/

read_cir(filename,in,Box,cir,read_header)
char *filename;
FILE *in;
Box_info *Box;  
Cir *cir; 
int read_header;
/*************************************************************/
/* Input the needed info for an em analysis.                 */
/* Returns:						     */
/*	1  - bad key					     */
/*	0  - read file properly & good key		     */
/*	-3 - No file present				     */
/*	-2 - unexpected end of file.			     */
/*	-1 - .geo file is of improper format		     */
/* (C) COPYRIGHT 1991 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr23may87 */ 
/* jcr13may89 modified for polygonal outline circuit description*/
{ int i,npoly;
  char Str[256],*Pos,Version[16];
  int rc;
  int poly_index, lev_index;

  /* The Box variable must have been initialized by init_box b4 calling 
   * this proc.
   */

  /* Read in circuit header information */
  rc = Get_circuit_head(filename,in,Box,cir,Version,Str);
  if ( rc != 0 ) return rc;

  /* check if only header information is being read */
  if ( read_header ) return( 0 );

  /*Load the circuit geometry*/
  Pos=Skip_blnks(Str);
  while( strncmp(Pos,"NUM",3)!=0 )
  { if (sFgets(Str,255,in)==NULL)
    { fprintf(stderr,"File \"%s\", line %d: NUM line missing.",filename,Fgets_line_num() );
      return(-1);
    }
    Pos=Skip_blnks(Str);
  }
  npoly = atoi(Next_field(Pos));
  if (npoly<0)
  { fprintf(stderr,"File \"%s\", line %d: NUM line indicates negitive number of polygons.\n",
	filename,Fgets_line_num() );  
    return(-1); 
  }

  /*Load the circuit polygons*/
  for (i=0; i<npoly; i++)
  { 
    rc = readpoly(&poly_index,&lev_index,filename,Box,cir,in,0,0,0,1.0);
    if ( rc != 0 ) {
	return(rc);
    }
  }

  return(0);

} /*read_cir*/

#ifdef NO_EM_SECURITY

	/************************************************
	*						*
	*		NO SECURITY IMPLEMENTED		*
	*						*
	************************************************/

int
get_sub_limit()
{
	return( 0 );	/* No subsection limit */
}

int
check_security(prog_id) 
int prog_id;
{
	good_key = 1;
	return( 1 );
}

check_sub_limit(sub,file,Com)
int sub;
FILE *file;
char *Com;
{
	return (0);
}

char *
get_license_id()
{
	return("UNDEFINED_LICENSE");
}

char *
get_node_id()
{
	return("UNDEFINED_NODE");
}

print_sn(file)
FILE *file;
{
	fprintf(file,"Serial number:\tUNDEFINED_SERIAL_NUMBER");
}

#else


#include "security.i"

/********************/
/* 		    */
/* Used in security */
/* 		    */
/********************/

Key key;

/*Note: Get_circuit is start and init_met is end point for check_security cksum*/
check_security(prog_id) 
int prog_id;
/******************************************************************/
/* Check to see if the time limit on the program has expired.     */
/* Or if the software is node locked				  */
/* Copyright (c) 1989 Sonnet Software, Inc., All rights reserved. */
/******************************************************************/
/* 20Sep90krs */
{
  int security();
  int rc;
  extern char *get_release();
  rc = security(prog_id, get_release(), -1, (char *)NULL );
  return (rc);

} /*check_security*/


/************************************************************
	Check to see if the user has exceeded the subsection 
	limit.
	CASES:
		- No limit -> return
		- Limit, but users is analyzing a circuit with 
		  less subsections than the limit -> return
		- Use has exceeded their limit -> print out message
		  and stop
		- invalid keyfile and subs <= 120 -> print out message
		  and continue
		- invalid keyfile and subs > 120 -> print out message
		  and stop
************************************************************/
#define PRTMSG(x) printf(x); fprintf(file,"%s ",Com); fprintf(file,x);

check_sub_limit(sub,file,Com)
int sub;
FILE *file;
char *Com;
{
	int sub_limit;
	char outstr[80];

	if ( good_key ) {
		sub_limit = get_sub_limit();
	} else {
		sub_limit = -1;
	}

	if ( sub_limit == 0 ) {
		return;
	}

	if  (sub_limit>0) {
		if ( sub<=sub_limit ) {
			return;
		} else {
			if ( sub_limit <= EM_SUB_LIMIT ) {
				/* user exceeded demo limit */
				PRTMSG("You have exceeded the demo subsection limit.\n");
				sprintf(outstr,"Your subsection limit is %d.\n",sub_limit);
				PRTMSG(outstr);
				PRTMSG("You should either reduce the size of the circuit,\n");
				PRTMSG("remove metal from the circuit or reduce the resolution\n");
				PRTMSG("of the circuit.\n");
				exit_program (1);
			} else {
				/* User exceeded their limit */
				PRTMSG("You have exceeded your subsection limit.\n");
				sprintf(outstr,"Your subsection limit is %d.",sub_limit);
				PRTMSG(outstr);
				exit_program (1);
			}
		}
	}
}

#endif	/* NO_EM_SECURITY */

/* Print warning message at the beginning of the cir.rsp file */
subsection_warning(file,Com,print_license_warning)
FILE *file;
char *Com;
int print_license_warning;
{

	int sub_limit;

	if ( good_key ) {
		sub_limit = get_sub_limit();
		if ( sub_limit == 0 ) return;
	} else {
		sub_limit = -1;
	}

	if ( ! print_license_warning ) return;	/* User used the -Q option */
	if ( sub_limit > EM_SUB_LIMIT ) return; /* Not demo */

        fprintf(file,"%s----------------------------------------------------------------------\n",Com);
        fprintf(file,"%s Commercial or for-profit users of this program must first pay Sonnet\n",Com);
        fprintf(file,"%s Software, Inc. the appropriate license fee. No fee is required for private\n",Com);
        fprintf(file,"%s or education use for circuits under 120 subsections.  Any one may use\n",Com);
        fprintf(file,"%s this program for a free one month trial.  All user sites (including free\n",Com);
        fprintf(file,"%s sites) must register with Sonnet within one week of initial use.\n",Com);

        fprintf(file,"%s If you agree to these terms and do not want this message displayed again,\n",Com);
        fprintf(file,"%s use the \"-Q\" option.\n",Com);
        fprintf(file,"%s Example: em -Qvdm filter.geo.\n",Com);
        fprintf(file,"%s----------------------------------------------------------------------\n",Com);

} /*subsection_warning*/


clean_box(Box)  Box_info *Box;
/*************************************************************/
/* Free up all memory occupied by Box except for Box itself  */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr06jan88 */ 
{ int i;
  if (Box->file) Box->file = NULL;  /*Ptr not ness. malloced, do not Free*/
  if (Box->H) sFree( (char*)Box->H);
  if (Box->Eps) sFree( (char*)Box->Eps);
  if (Box->Mu) sFree( (char*)Box->Mu);
  if (Box->Eloss) sFree( (char*)Box->Eloss);
  if (Box->Mloss) sFree( (char*)Box->Mloss);
  for (i=0; i<Box->num_met; i++) if ( (Box->met+i)->name ) sFree(  (char*)(Box->met+i)->name );
  if (Box->met) sFree( (char*)Box->met);
  if (Box->Top.name) sFree( (char*)Box->Top.name);
  if (Box->Bot.name) sFree( (char*)Box->Bot.name);
  for (i=0; i<NUM_ESUB; i++) clean_sub(Box->sub+i);
  if (Box->sub) sFree( (char*)Box->sub);
  clear_ann(&(Box->ann),&(Box->num_ann));
  if (Box->ej_loss) sFree( (char*)Box->ej_loss);
  init_box(Box);
} /*clear_box*/

clear_ann(ann_arrp,nump)  char ***ann_arrp;  int *nump;
/*************************************************************/
/* Add a comment to the array of comments.                   */
/* (C) COPYRIGHT 1986 JAMES C. RAUTIO.  ALL RIGHTS RESERVED. */
/*************************************************************/
/* jcr01jan88 */ 
{ while (*nump>0) sFree( *(*ann_arrp + (--(*nump)) ) );
  if (*ann_arrp != NULL) sFree( (char *)*ann_arrp);
  *ann_arrp = NULL;
  *nump = 0;
} /*clear_ann*/

